# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from rains.common.decorator import singleton_pattern


@singleton_pattern
class CaseRecorder(object):
    """
    [ 用例记录器 ]
    * 该模块负责记录用例执行时产生的日志，并且暂存至下一次读取。

    """

    _record_dict: dict
    """ 记录列表字典 """

    def __init__(self):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._record_dict = {}

    @property
    def record_dict(self):

        return self._record_dict

    def write(self, core_id: int, record: str):
        """
        [ 录入 ]
        * 无

        [ 必要参数 ]
        * core_id (int) : 核心ID
        * record (str) : 录入的信息

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if core_id in self.record_dict:
            self.record_dict.update({core_id: ''.join([self.record_dict[core_id], '\n', record])})

        else:
            self.record_dict[core_id] = record

    def read(self, core_id) -> str:
        """
        [ 读取 ]
        * 读取后，缓冲器会清空所有暂存数据。

        [ 必要参数 ]
        * core_id (int) : 核心ID
        * record (str) : 录入的信息

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 所有记录拼接成的字符串

        """

        return_record_str = self._record_dict[core_id]

        del self._record_dict[core_id]

        return return_record_str



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from rains.common.db.sql_packaging.sql_task import SqlTask
from rains.common.db.sql_packaging.sql_case import SqlCase


class SqlLang(object):
    """
    [ SQL语句 ]
    * 动态拼接 SQLite 语句。

    """

    @property
    def task(self):
        """
        [ 任务相关的 SQL 语句 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (SqlTask) : 任务相关的 SQL 语句封装类

        """

        return SqlTask

    @property
    def case(self):
        """
        [ 用例相关的 SQL 语句 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (SqlCase) : 用例相关的 SQL 语句封装类

        """

        return SqlCase



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from abc import ABCMeta
from abc import abstractmethod


class IEngine(metaclass=ABCMeta):
    """
    [ 引擎接口 ]
    * 无

    """

    @abstractmethod
    def get_state(self) -> bool:
        """
        [ 获取引擎状态 ]
        * 获取当前引擎状态，启动中返回 True，否则返回 False。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (Bool) : 当前引擎状态

        """

    @abstractmethod
    def run(self):
        """
        [ 运行引擎 ]
        * 驱动与定位器的实例化。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

    @abstractmethod
    def end(self):
        """
        [ 结束引擎 ]
        * 退出并销毁当前持有的驱动与定位器。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

    @abstractmethod
    def reset(self):
        """
        [ 重置引擎 ]
        * 将引擎重置至初始阶段。
        * 与 end() 相比，该函数只是重置引擎至初始状态，而不是注销。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = ['SqlCase']


from rains.common.db.sql_packaging.common import *


class SqlCase(object):
    """
    [ 用例相关的 SQL 语句 ]
    * 无
    
    """

    @staticmethod
    def get_count_all() -> str:
        """
        [ 查询所有用例数量 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句
        
        """
        
        return f""" 

        SELECT COUNT(id) FROM { ConstDatabaseTableNaming.CASES } 

        """

    @staticmethod
    def get_count_pass() -> str:
        """
        [ 查询所有通过的用例数量 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句
        
        """
        
        return f""" 

        SELECT COUNT(id) FROM { ConstDatabaseTableNaming.CASES }

        WHERE { ConstDatabaseCaseNaming.STATE } == '{ ConstTaskAndCaseState.SUCCESSFUL }'

        """

    @staticmethod
    def get_count_fail() -> str:
        """
        [ 查询所有状态为非'成功'用例数量 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句
        
        """
        
        return f""" 

        SELECT COUNT(id) FROM { ConstDatabaseTableNaming.CASES }

        WHERE { ConstDatabaseCaseNaming.STATE } != '{ ConstTaskAndCaseState.SUCCESSFUL }'

        """

    @staticmethod
    def get_count_from_data(paras: dict) -> str:
        """
        [ 查询指定日期里所有失败的用例数量 ]
        * 无

        [ 必要参数 ]
        * execute (str) : 执行日期

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseCaseNaming.EXECUTE_DATE
            ]
        )

        return f""" 

        SELECT COUNT(id) FROM { ConstDatabaseTableNaming.CASES }

        WHERE { ConstDatabaseCaseNaming.EXECUTE_DATE } == '{ paras[ConstDatabaseCaseNaming.EXECUTE_DATE] }'

        """

    @staticmethod
    def get_count_fail_from_data(paras: dict) -> str:
        """
        [ 查询指定日期里所有失败的用例数量 ]
        * 无

        [ 必要参数 ]
        * execute (str) : 执行日期

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句
        
        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseCaseNaming.EXECUTE_DATE
            ]
        )

        return f""" 

        SELECT COUNT(id) FROM { ConstDatabaseTableNaming.CASES }

        WHERE { ConstDatabaseCaseNaming.STATE } != '{ ConstTaskAndCaseState.SUCCESSFUL }'

        AND { ConstDatabaseCaseNaming.EXECUTE_DATE } == '{ paras[ConstDatabaseCaseNaming.EXECUTE_DATE] }'

        """

    @staticmethod
    def get_count_from_tid(paras: dict) -> str:
        """
        [ 查询指定任务下所有用例数量 ]
        * 无

        [ 必要参数 ]
        * tid (int) : 所属任务ID

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseTaskNaming.TID
            ]
        )

        return f""" 

        SELECT COUNT(id) FROM { ConstDatabaseTableNaming.CASES } 

        WHERE { ConstDatabaseTaskNaming.TID } = { paras[ConstDatabaseTaskNaming.TID] }

        """

    @staticmethod
    def get_count_form_date(paras: dict) -> str:
        """
        [ 查询指定执行日期的用例数量 ]
        * 无

        [ 必要参数 ]
        * execute (str) : 执行日期

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseTaskNaming.EXECUTE_DATE
            ]
        )

        return f""" 

        SELECT COUNT(id) FROM { ConstDatabaseTableNaming.CASES } 

        WHERE { ConstDatabaseTaskNaming.EXECUTE_DATE } = '{ paras[ConstDatabaseTaskNaming.EXECUTE_DATE] }'

        """

    @staticmethod
    def get_info_from_tid(paras: dict) -> str:
        """
        [ 查询所有用例详情信息列表 ]
        * 无

        [ 必要参数 ]
        * tid (int) : 所属任务ID

        [ 可选参数 ]
        * state (str) : 状态
        * page (int) : 页数
        * number (int) : 查询数量

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseCaseNaming.TID
            ]
        )

        if ConstDatabaseCaseNaming.STATE in paras.keys():
            state = f'{ ConstDatabaseCaseNaming.STATE } = "{ paras[ConstDatabaseCaseNaming.STATE] }"'
        else:
            state = f'{ ConstDatabaseCaseNaming.STATE } in ("{ ConstTaskAndCaseState.BLOCK }", ' \
                    f'"{ ConstTaskAndCaseState.SUCCESSFUL }", "{ ConstTaskAndCaseState.UNSUCCESSFUL }")'

        return f"""

        SELECT * FROM { ConstDatabaseTableNaming.CASES }

        WHERE { state } AND { ConstDatabaseCaseNaming.TID } = { paras[ConstDatabaseCaseNaming.TID] }

        ORDER BY ID DESC

        { DatabaseParameter.get_desc_limit(paras) }

        """

    @staticmethod
    def get_info_from_cid(paras: dict) -> str:
        """
        [ 查询指定CID的用例详情信息 ]
        * 无

        [ 必要参数 ]
        * cid (int) : 用例ID

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseCaseNaming.CID
            ]
        )

        return f"""

        SELECT * FROM { ConstDatabaseTableNaming.CASES }

        WHERE id = { paras[ConstDatabaseCaseNaming.CID] }

        """

    @staticmethod
    def add(paras: dict) -> str:
        """
        [ 创建用例 ]
        * 无

        [ 必要参数 ]
        * tid (int) : 所属任务记录ID
        * name (str) : 用例名称
        * remark (str) : 用例备注
        * execute_date (str) : 执行日期

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseCaseNaming.TID,
                ConstDatabaseCaseNaming.NAME,
                ConstDatabaseCaseNaming.REMARK,
                ConstDatabaseCaseNaming.EXECUTE_DATE
            ]
        )

        return f""" 

        INSERT INTO { ConstDatabaseTableNaming.CASES } (

            id, 
            { ConstDatabaseCaseNaming.TID }, 
            { ConstDatabaseCaseNaming.NAME }, 
            { ConstDatabaseCaseNaming.REMARK }, 
            { ConstDatabaseCaseNaming.STATE }, 
            { ConstDatabaseCaseNaming.EXECUTE_DATE }, 
            { ConstDatabaseCaseNaming.START_TIME }, 
            { ConstDatabaseCaseNaming.END_TIME },
            { ConstDatabaseCaseNaming.SPEND_TIME_S },
            { ConstDatabaseCaseNaming.RUN_STEP }
        )

        VALUES (

            NULL,
            '{ paras[ConstDatabaseCaseNaming.TID] }',
            '{ paras[ConstDatabaseCaseNaming.NAME] }',
            '{ paras[ConstDatabaseCaseNaming.REMARK] }',
            '{ ConstTaskAndCaseState.BLOCK }', 
            '{ paras[ConstDatabaseCaseNaming.EXECUTE_DATE] }',
             NULL, 
             NULL,
             NULL,
             NULL
        )

        """

    @staticmethod
    def update(paras: dict) -> str:
        """
        [ 更新用例 ]
        * 无

        [ 必要参数 ]
        * cid (int) : 用例ID
        * state (str) : 用例状态
        * start_time (date) : 开始时间
        * end_time (date) : 结束时间
        * spend_time_s (int) : 消耗时间(秒)
        * run_step (str) : 运行步骤

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseCaseNaming.CID,
                ConstDatabaseCaseNaming.STATE,
                ConstDatabaseCaseNaming.START_TIME,
                ConstDatabaseCaseNaming.END_TIME,
                ConstDatabaseCaseNaming.SPEND_TIME_S,
                ConstDatabaseCaseNaming.RUN_STEP
            ]
        )

        return f"""

        UPDATE { ConstDatabaseTableNaming.CASES }

        SET 
            { ConstDatabaseCaseNaming.STATE }         = '{ paras[ConstDatabaseCaseNaming.STATE] }',
            { ConstDatabaseCaseNaming.START_TIME }    = '{ paras[ConstDatabaseCaseNaming.START_TIME] }',
            { ConstDatabaseCaseNaming.END_TIME }      = '{ paras[ConstDatabaseCaseNaming.END_TIME] }',
            { ConstDatabaseCaseNaming.SPEND_TIME_S }  = '{ paras[ConstDatabaseCaseNaming.SPEND_TIME_S] }',
            { ConstDatabaseCaseNaming.RUN_STEP }      = '{ paras[ConstDatabaseCaseNaming.RUN_STEP] }'

        WHERE
            ID = { paras[ConstDatabaseCaseNaming.CID] }

        """

    @staticmethod
    def delete_from_cid(paras: dict) -> str:
        """
        [ 删除指定CID用例 ]
        * 无

        [ 必要参数 ]
        * cid (int) : 用例ID

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras, 
            [
                ConstDatabaseCaseNaming.CID
            ]
        )

        return f"""

        DELETE FROM { ConstDatabaseTableNaming.CASES }

        WHERE ID = { paras[ConstDatabaseCaseNaming.CID] }

        """

    @staticmethod
    def delete_from_tid(paras: dict = None) -> str:
        """
        [ 删除指定TID所有用例 ]
        * 无

        [ 必要参数 ]
        * tid (int) : 所属任务ID

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseTaskNaming.TID
            ]
        )

        return f"""

        DELETE FROM { ConstDatabaseTableNaming.CASES }

        WHERE TID = { paras[ConstDatabaseTaskNaming.TID] }

        """

    @staticmethod
    def delete_all() -> str:
        """
        [ 删除所有用例 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        return f"""

        DELETE FROM { ConstDatabaseTableNaming.CASES }

        """



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


import time
import random

from rains.common.log import Log
from rains.common.db.database import Database
from rains.common.db.sql_lang import SqlLang
from rains.common.db.const import ConstDatabaseTaskNaming
from rains.common.db.const import ConstDatabaseCaseNaming

from rains.common.const import ConstProjectType
from rains.common.const import ConstTaskAndCaseState
from rains.common.const import ConstTaskOrderSignNaming

from rains.common.i.i_core import ICore
from rains.common.task.case_recorder import CaseRecorder
from rains.common.task.task_instruction import TaskInstruction

from rains.kit.mini.mini_plant import MiniPlant
from rains.kit.mini.mini_engine import MiniEngine


_CORE_NUMBER_POOL = []
""" 核心编号池 """


class MiniCore(ICore):
    """
    [ 小程序核心 ]
    *  运行核心相对于任务的消费者，它负责接收任务，并解析运行。
    *  另外，它代理着一个 MiniEngine 引擎对象，并在工作流中管理其生命周期。

    """

    _log = Log()
    """ 日志服务 """

    _db = Database()
    """ 数据库服务 """

    _sql: SqlLang = SqlLang()
    """ SQL语句封装 """

    _mini_engine: MiniEngine = None
    """ 引擎对象 """

    _mini_plant: MiniPlant = None
    """ 工厂对象 """

    _core_id: int
    """ 核心ID """

    _project_type: str = ConstProjectType.MINI
    """ 项目类型 """

    _case_recorder: CaseRecorder = CaseRecorder()
    """ 用例记录器 """

    def __init__(self, project_path: str, wx_cli_path: str):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * project_path (str) : 项目路径
        * wx_cli_path (str) : 微信开发者工具 cli.bat 的路径

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        # 生成核心ID并创建引擎对象
        while True:

            self._core_id = random.randint(10000, 99999)

            if self._core_id not in _CORE_NUMBER_POOL:
                _CORE_NUMBER_POOL.append(self._core_id)
                break

            else:
                continue

        self._mini_engine = MiniEngine(project_path, wx_cli_path, self._core_id)

    @property
    def engine(self):
        return self._mini_engine

    @property
    def plant(self):
        return self._mini_plant

    @property
    def core_id(self):
        return self._core_id

    @property
    def project_type(self):
        return self._project_type

    def _log_output(self, message: str, log: bool):
        """
        [ 日志输出 ]
        * 无

        [ 必要参数 ]
        * message (str) : 输出信息

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if log:
            self._log.info(f'CASE_OK::[执行核心::{ self._mini_engine.core_id }]::{ message } \n')

    def start_task(self, task, log: bool = True):
        """
        [ 执行 task ]
        * 接收一个 task，并执行该 task。

        [ 必要参数 ]
        * task (object) : 任务对象，可以是实现 ITask 接口的任务类，或者是符合JSON译文标准的字典

        [ 可选参数 ]
        * log (bool) : 日志开关

        [ 返回内容 ]
        * 无

        """

        # 执行任务类
        if not isinstance(task, dict):
            return self._running_task_code(TaskInstruction().convert_from_code(task), log)

        else:
            pass

    def end(self):
        """
        [ 注销 ]
        * 关闭数据库连接，并调用引擎对象的 end() 函数。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._db.close()
        self._mini_engine.end()

        return self

    def _running_task_code(self, task_instruction: TaskInstruction, log: bool):
        """
        [ 执行 CODE 译文 ]
        * 无

        [ 必要参数 ]
        * task_instruction (Object) : TaskInstruction 对象

        [ 可选参数 ]
        * log (bool) : 日志开关

        [ 返回内容 ]
        * 无

        """

        # 实例化任务类型
        task_class_living = task_instruction.task_class()
        task_class_living.engine = self._mini_engine
        task_class_living.plant = self._mini_plant

        if not task_class_living.engine.get_state():
            task_class_living.engine.run()

        # 初始化数据缓冲区
        db_task_id = 0
        execute_data = time.strftime("%Y-%m-%d")
        case_all_count = 0
        case_pass_count = 0
        case_fail_count = 0
        task_is_completed = True
        task_start_time = ''
        task_spend_time_s = 0

        # 解析步骤指令集数量
        all_case_count = len(task_instruction.instruction_map_step.keys())
        # 声明当前用例完成进度
        complete_count = 1

        try:
            # 获取任务注释内容
            remark = 'NULL'
            if type(task_class_living).__doc__:
                remark = type(task_class_living).__doc__.strip()

            # 创建数据库任务记录
            db_task_id = self._db.write(self._sql.task.add({
                ConstDatabaseTaskNaming.NAME: task_instruction.task_name,
                ConstDatabaseTaskNaming.REMARK: remark,
                ConstDatabaseTaskNaming.EXECUTE_DATE: execute_data
            }))

            # 执行 类级起点 函数
            self._running_instruction_map_flow(
                task_instruction.instruction_map_flow[ConstTaskOrderSignNaming.CLASS_STARTING],
                task_class_living
            )

            # 记录任务开始时间
            task_start_time = time.strftime("%H:%M:%S")
            task_spend_time_s = int(time.time())

            # 执行步骤指令集
            for case_name, case_function in task_instruction.instruction_map_step.items():

                if log:
                    self._log.info(f'— case running ->>> [{all_case_count}/{complete_count}]')

                # 执行 函数级起点 函数
                self._running_instruction_map_flow(
                    task_instruction.instruction_map_flow[ConstTaskOrderSignNaming.FUNCTION_STARTING],
                    task_class_living
                )

                # 获取用例函数注释内容
                case_remark = 'NULL'
                if case_function.__doc__:
                    case_remark = case_function.__doc__.strip()

                # 数据库创建用例记录
                db_case_id = self._db.write(self._sql.case.add({
                    ConstDatabaseCaseNaming.TID: db_task_id,
                    ConstDatabaseCaseNaming.NAME: case_name,
                    ConstDatabaseCaseNaming.REMARK: case_remark,
                    ConstDatabaseCaseNaming.EXECUTE_DATE: execute_data
                }))

                # 用例计数器自增
                case_all_count += 1

                running_result = self._running_case(db_case_id, task_class_living, case_function)

                if running_result:
                    case_pass_count += 1
                    self._log_output(f'case[{ case_name }]::[{ConstTaskAndCaseState.SUCCESSFUL}]', log)
                else:
                    case_fail_count += 1
                    self._log_output(f'case[{ case_name }]::[{ConstTaskAndCaseState.UNSUCCESSFUL}]', log)
                    task_is_completed = False

                # 当前用例完成进度自增
                complete_count += 1

                # 执行 函数级终点 函数
                self._running_instruction_map_flow(
                    task_instruction.instruction_map_flow[ConstTaskOrderSignNaming.FUNCTION_ENDING],
                    task_class_living
                )
                self._mini_engine.reset()

            # 执行 类级终点 函数
            self._running_instruction_map_flow(
                task_instruction.instruction_map_flow[ConstTaskOrderSignNaming.CLASS_ENDING],
                task_class_living
            )

            # 如果引擎仍在运行，则结束引擎
            if task_class_living.engine.get_state():
                task_class_living.engine.end()

        except BaseException as e:
            self._log_output(f'由于未知异常任务被迫中止:: { e }', log)
            task_class_living.engine.end()

        finally:
            # 更新数据库任务记录
            task_end_time = time.strftime("%H:%M:%S")
            self._db.write(self._sql.task.update({
                ConstDatabaseTaskNaming.TID: db_task_id,
                ConstDatabaseTaskNaming.IS_COMPLETED: task_is_completed,
                ConstDatabaseTaskNaming.START_TIME: task_start_time,
                ConstDatabaseTaskNaming.END_TIME: task_end_time,
                ConstDatabaseTaskNaming.SPEND_TIME_S: (int(time.time()) - task_spend_time_s),
                ConstDatabaseTaskNaming.CASE_ALL: case_all_count,
                ConstDatabaseTaskNaming.CASE_PASS: case_pass_count,
                ConstDatabaseTaskNaming.CASE_FAIL: case_fail_count
            }))
            self._db.commit()

    def _running_case(self, case_id: int, task_class, case_function) -> bool:
        """
        [ 执行用例 ]
        * 无

        [ 必要参数 ]
        * case_id (int): 用例数据库记录ID
        * task_class (object): 任务类型实例
        * case_function (function): 用例函数

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (bool) 是否完成布尔值

        """
        case_db_dict = dict()
        case_db_dict[ConstDatabaseCaseNaming.CID] = case_id
        case_start_time = time.strftime("%H:%M:%S")
        case_spend_time_s = int(time.time())

        try:
            # 执行用例
            r = case_function(task_class)

            # 如果用例返回 True 或者未返回 False 都判断为用例通过
            if r is True or r is None:
                case_db_dict[ConstDatabaseCaseNaming.STATE] = ConstTaskAndCaseState.SUCCESSFUL
                return True

            # 如果用例返回 False 则判断用例失败
            else:
                case_db_dict[ConstDatabaseCaseNaming.STATE] = ConstTaskAndCaseState.UNSUCCESSFUL
                return False

        # 如果用例执行过程发生异常则判断用例失败
        except BaseException as e:
            case_db_dict[ConstDatabaseCaseNaming.STATE] = ConstTaskAndCaseState.ANOMALY
            self._log.debug(f'{ e }')
            return False

        finally:
            case_db_dict[ConstDatabaseCaseNaming.START_TIME] = case_start_time
            case_db_dict[ConstDatabaseCaseNaming.END_TIME] = time.strftime("%H:%M:%S")
            case_db_dict[ConstDatabaseCaseNaming.SPEND_TIME_S] = (int(time.time()) - case_spend_time_s)
            case_db_dict[ConstDatabaseCaseNaming.RUN_STEP] = self._case_recorder.read(self.core_id)

            self._db.write(self._sql.case.update(case_db_dict))

    @staticmethod
    def _running_instruction_map_flow(function, t):
        if function:
            function(t)



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = ['MiniEngine']


import minium

from minium import Minium

from rains.common.log import Log
from rains.common.const import ConstProjectType
from rains.common.i.i_engine import IEngine

from rains.kit.mini.packaging.mini_find_handle import MiniFindHandle


class MiniEngine(IEngine):
    """
    [ 小程序引擎 ]
    * 引擎封装了原生驱动的获取逻辑与小程序元素定位器，并且管理着驱动对象的生命周期。

    """

    _log = Log()
    """ WEB引擎 """

    _state: bool
    """ 状态 """

    _core_id: int
    """ 编号 """

    _type: str = ConstProjectType.MINI
    """ 类型 """

    _driver: Minium = None
    """ 原生驱动对象 """

    _find_handle: MiniFindHandle = None
    """ 元素定位器 """

    driver_config: dict = {

        # 小程序运行的平台，可选值为: ide, Android, IOS
        'platform': 'ide',

        # 日志打印级别，可选: error, warn, info, debug
        'debug_mode': 'error',

        # IDE监听的端口，默认为 9420
        'test_port': 9420,

        # 请求连接超时时间 s
        'request_timeout': 30,

        # 每个case启动的时候relaunch到启动页面
        'auto_relaunch': True,

        # 自动处理授权弹窗
        # 包括: 获取用户信息，获取位置，获取微信运动数据，获取录音权限，获取相册权限，获取摄像头权限
        'auto_authorize': False

    }
    """ 驱动配置 """

    def __init__(self, project_path: str, wx_cli_path: str, core_id: int):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * project_path (str) : 项目路径
        * wx_cli_path (str) : 微信开发者工具 cli.bat 的路径
        * core_id (int) : 核心ID，默认为空，不传则自动生成

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 小程序引擎对象

        """

        self.driver_config['project_path'] = project_path
        self.driver_config['dev_tool_path'] = wx_cli_path
        self._core_id = core_id
        self.state = False

    @property
    def core_id(self):
        return self._core_id

    @property
    def type(self):
        return self._type

    @property
    def driver(self):
        return self._driver

    @property
    def find_handle(self):
        return self._find_handle

    def get_state(self) -> bool:
        """
        [ 获取引擎状态 ]
        * 获取当前引擎状态，启动中返回 True，否则返回 False。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (Bool) : 当前引擎状态

        """

        return self.state

    def run(self):
        """
        [ 运行引擎 ]
        * 驱动与定位器的实例化。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if not self.state:

            self._log.info('正在尝试建立"微信开发者工具"的服务端口连接...')

            self._driver = minium.Minium(self.driver_config)

            self._log.info('成功建立"微信开发者工具"的服务端口连接...')
            self._log.info(f'打印当前"微信开发者工具"的系统信息:: { self._driver.get_system_info() }')

            self._find_handle = MiniFindHandle(self._driver)
            self.state = True

        return self

    def end(self):
        """
        [ 结束引擎 ]
        * 退出并销毁当前持有的驱动与定位器。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if self.state:

            self._driver.shutdown()

            del self._driver

            self.state = False

            return self

    def reset(self):
        """
        [ 重置引擎 ]
        * 将引擎重置至初始阶段。
        * 与 end() 相比，该函数只是重置引擎至初始状态，而不是注销。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if self.state:

            self._driver.app.go_home()

        return self



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = ['MiniPlantElement']


from rains.common.log import Log

from rains.kit.mini.mini_engine import MiniEngine
from rains.kit.mini.packaging.mini_element_structure import MiniElementStructure


class MiniPlantElement(object):
    """
    [ 小程序元素控件 ]
    * 基于元素特征构建指向该元素的元素控件对象。

    """

    _log = Log()
    """ [ 日志服务 ] """

    _mini_engine: MiniEngine
    """ [ 引擎对象 ] """

    _element_structure: MiniElementStructure
    """ [ 元素结构体 ] """

    _element_container: list = []
    """ [ 元素列表容器 ] """

    def __init__(self,
                 mini_engine: MiniEngine,
                 mini_element_structure: MiniElementStructure):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * mini_engine (MiniEngine): 小程序引擎对象
        * mini_element_structure (MiniElementStructure): 小程序元素结构体

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._mini_engine = mini_engine
        self._element_structure = mini_element_structure

    def _log_output(self, message: str, log: bool):
        """
        [ 日志输出 ]
        * 无

        [ 必要参数 ]
        * message (str) : 输出信息
        * log (bool) : 输出判断值

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if log:
            self._log.info(f'执行核心[{ self._mini_engine.id }]::'
                           f'{ self._element_structure.page }::'
                           f'{ self._element_structure.name }::'
                           f'{ message }')

    def _err_output(self, message: str):
        """
        [ 错误输出 ]
        * 无

        [ 必要参数 ]
        * message (str) : 输出信息

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        raise Exception((f'执行核心[{ self._mini_engine.id }]::'
                         f'{ self._element_structure.page }::'
                         f'{ self._element_structure.name }::'
                         f'{ message }'))

    def _analytical_elements(self, wait: int = 15) -> list:
        """
        [ 解析函数 ]
        * 每一次调用页面元素组件的方法时，都会调用该函数，以更新定位结果，确认组件是否存在于当前页面中。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * wait (int) : 查找元素所等待的显式等待时长

        [ 返回内容 ]
        * 元素列表容器

        """

        try:
            self._element_container = \
                self._mini_engine.find_handle.find(self._element_structure, wait)

            return self._element_container

        except BaseException as e:
            self._err_output(f'元素控件精灵类解析异常:: { e } !')

    @property
    def get(self):
        """
        [ 小程序元素控件'获取'相关函数 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        return _MiniPlantElementMapGet(self._mini_engine,
                                       self._analytical_elements,
                                       self._log_output,
                                       self._err_output)

    @property
    def mouse(self):
        """
        [ 小程序元素控件'鼠标'相关函数 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        return _MiniPlantElementMapMouse(self._mini_engine,
                                         self._analytical_elements,
                                         self._log_output,
                                         self._err_output)

    @property
    def input(self):
        """
        [ 小程序元素控件'输入框'相关函数 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        return _MiniPlantElementMapInput(self._mini_engine,
                                         self._analytical_elements,
                                         self._log_output,
                                         self._err_output)

    def switch(self, eid: int = 0, log: bool = True):
        """
        [ 改变 switch 组件的状态 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回内容 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()

            self._element_container[eid].switch()
            self._log_output(f'改变 switch 组件的状态', log)

            return self

        except BaseException as err:
            self._err_output(f'改变 switch 组件的状态异常::{ err }')

    def slide_to(self, value: int, eid: int = 0, log: bool = True):
        """
        [ slider 组件滑动到指定数值 ]
        * 无

        [ 必要参数 ]
        * value (int) : 数值

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回内容 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()

            self._element_container[eid].slide_to(value)
            self._log_output(f'slider 组件滑动到指定数值[{ value }]', log)

            return self

        except BaseException as e:
            self._err_output(f'slider 组件滑动到指定数值异常:: { e }')

    def pick(self, value: int or str, eid: int = 0, log: bool = True):
        """
        [ picker 选择器组件选值 ]
        * value 的取值:
        * selector / multiSelector / region 选择器需要传递 int ，表示选择了 range 中的第几个 (下标从 0 开始)
        * time 时间选择器需要传递 str，表示选中的时间，格式为 "hh:mm"，如 "9:27"
        * date 日期选择器需要传递 str，表示选中的日期，格式为"YYYY-MM-DD"，如 "2020-07-07"

        [ 必要参数 ]
        * value (int or str) : 选择器取值

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回内容 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()

            self._element_container[eid].pick(value)
            self._log_output(f'选择器选值 -> [{ value }]', log)

            return self

        except BaseException as e:
            self._err_output(f'选择器选值异常::{ e }')


class _MiniPlantElementMapBase(object):
    """
    [ WEB元素控件封装基类 ]
    * 无

    """

    _mini_engine: MiniEngine
    """ [ 引擎对象 ] """

    _element_container: list = []
    """ [ 元素列表容器 ] """

    _analytical_elements = None
    """ [ 解析函数 ] """

    _log_output = None
    """ [ 日志输出函数 ] """

    _err_output = None
    """ [ 错误输出函数 ] """

    def __init__(self, mini_engine: MiniEngine,
                 analytical_elements_function,
                 log_output_function,
                 err_output_function):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * mini_engine (MiniEngine) : 小程序引擎对象
        * analytical_elements_function (function): 元素解析函数
        * log_output_function (function): 日志输出函数
        * err_output_function (function): 错误输出函数

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._web_engine = mini_engine
        self._analytical_elements = analytical_elements_function
        self._log_output = log_output_function
        self._err_output = err_output_function


class _MiniPlantElementMapGet(_MiniPlantElementMapBase):
    """
    [ 小程序元素控件'获取'相关函数 ]
    * 无

    """

    def count(self, log: bool = True) -> int:
        """
        [ 获取元素数量 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * log (bool) : 日志开关

        [ 返回内容 ]
        * (int) : 元素数量

        """

        try:
            self._element_container = self._analytical_elements()

            count = len(self._element_container)
            self._log_output(f'获取元素列表数量[{ count }]', log)

            return count

        except BaseException as e:
            self._err_output(f'获取元素列表数量异常:: { e }')

    def text(self, eid: int = 0, log: bool = True) -> str:
        """
        [ 获取文本 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回内容 ]
        * (str) : 元素文本

        """

        try:
            self._element_container = self._analytical_elements()

            text = self._element_container[eid].inner_text
            self._log_output(f'获取文本[{ text }]', log)

            return text

        except BaseException as e:
            self._err_output(f'获取文本异常:: { e }')

    def value(self, eid: int = 0, value_name: str = 'value', log: bool = True) -> str:
        """
        [ 获取属性值 ]
        * 获取元素名为 value_name 的属性值。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * value_name (str) : 获取属性的值名称
        * log (bool): 日志开关

        [ 返回内容 ]
        * (str) : 元素的属性值

        """

        try:
            self._element_container = self._analytical_elements()

            values = self._element_container[eid].attribute(value_name)
            self._log_output(f'获取属性值[{ values }]', log)

            return values

        except BaseException as e:
            self._err_output(f'获取属性值异常:: { e }')

    def styles(self, names: str or list, eid: int = 0, log: bool = True) -> str:
        """
        [ 获取样式属性 ]
        * 获取元素名为 names 的样式属性。

        [ 必要参数 ]
        * names (str or list) : 需要获取的 style 属性名称

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回内容 ]
        * (str) : 元素的样式属性值

        """

        try:
            self._element_container = self._analytical_elements()

            values = self._element_container[eid].styles(names)
            self._log_output(f'获取样式属性[{ values }]', log)

            return values

        except BaseException as e:
            self._err_output(f'获取样式属性异常:: { e }')

    def in_page(self, eid: int = 0, wait: int = 3, log: bool = True):
        """
        [ 判断元素是否存在当前可视页面 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * wait (int) : 查找元素所等待的显式等待时长
        * log (bool) : 日志开关

        [ 返回参数 ]
        * (bool) : 布尔值

        """

        try:
            self._element_container = self._analytical_elements(wait=wait)

            proof = False
            practical_id = eid + 1

            if len(self._element_container) >= practical_id:
                proof = True

            self._log_output(f'判断元素是否存在当前可视页面[{ proof }]', log)

            return proof

        except BaseException as e:
            self._err_output(f'判断元素是否存在当前可视页面异常:: { e }')


class _MiniPlantElementMapMouse(_MiniPlantElementMapBase):
    """
    [ 小程序元素控件'鼠标'相关函数 ]
    * 无

    """

    def tap(self, eid: int = 0, log: bool = True):
        """
        [ 左键点击 ]
        * 模拟一次作用于元素对象上的鼠标左键单击操作。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()

            self._element_container[eid].tap()
            self._log_output('执行[左键点击]', log)

            return self

        except BaseException as e:
            self._err_output(f'执行[左键点击]异常:: { e }')

    def long_tap(self, eid: int = 0, duration: int = 1000, log: bool = True):
        """
        [ 左键长按 ]
        * 模拟一次作用于元素对象上的鼠标左键长按操作。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * duration (int) : 长按时长，单位是毫秒，默认为 1000
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()

            self._element_container[eid].long_press(duration)
            self._log_output('执行[左键长按]', log)

            return self

        except BaseException as e:
            self._err_output(f'执行[左键长按]异常:: { e }')

    def move(self,
             x_offset: int,
             y_offset: int,
             move_delay: int = 500,
             smooth: bool = False,
             eid: int = 0,
             log: bool = True):
        """
        [ 鼠标拖动 ]
        * 模拟一次作用于元素对象上的鼠标拖动操作。

        [ 必要参数 ]
        * x_offset (int) : x 方向上的偏移，往右为正数，往左为负数
        * y_offset (int) : y 方向上的偏移，往右为正数，往左为负数

        [ 可选参数 ]
        * move_delay (int) : 移动前摇，单位为毫秒，默认为 500
        * smooth (bool) : 平滑移动，默认为 False
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()

            self._element_container[eid].move_to(
                x_offset=x_offset,
                y_offset=y_offset,
                move_delay=move_delay,
                smooth=smooth
            )

            self._log_output('执行[鼠标拖动]', log)

            return self

        except BaseException as e:
            self._err_output(f'执行[鼠标拖动]异常:: { e }')


class _MiniPlantElementMapInput(_MiniPlantElementMapBase):
    """
    [ 小程序元素控件'输入框'相关函数 ]
    * 无

    """

    def send(self, key: str, eid: int = 0, log: bool = True):
        """
        [ 输入内容 ]
        * 输入内容，一般指向 input & textarea 组件。

        [ 必要参数 ]
        * key (str) : 内容文本

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()

            self._element_container[eid].input(key)
            self._log_output(f'输入文本[{ key }]', log)

            return self

        except BaseException as e:
            self._err_output(f'输入文本异常:: { e }')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


class ConstDatabaseTableNaming(object):
    """
    [ 数据库表命名 ]
    * 无

    """

    TASKS = 'tasks'
    """ 任务表命名 """

    CASES = 'cases'
    """ 用例表命名 """


class ConstDatabaseTaskNaming(object):
    """
    [ 数据库任务表字段命名 ]
    * 无

    """

    TID = 'tid'
    """ 任务记录编号 """

    NAME = 'name'
    """ 任务名称 """

    REMARK = 'remark'
    """ 任务备注信息 """

    EXECUTE_DATE = 'execute_date'
    """ 执行日期 """

    IS_COMPLETED = 'is_completed'
    """ 是否已完成 """

    START_TIME = 'start_time'
    """ 开始时间 """

    END_TIME = 'end_time'
    """ 结束时间 """

    SPEND_TIME_S = 'spend_time_s'
    """ 消耗时间(秒) """

    CASE_ALL = 'case_all'
    """ 所有用例计数 """

    CASE_PASS = 'case_pass'
    """ 成功的用例计数 """

    CASE_FAIL = 'case_fail'
    """ 失败的用例计数 """


class ConstDatabaseCaseNaming(object):
    """
    [ 数据库用例表字段命名 ]
    * 无

    """

    CID = 'cid'
    """ 用例记录编号 """

    TID = 'tid'
    """ 所属的任务记录编号 """

    NAME = 'name'
    """ 用例名称 """

    REMARK = 'remark'
    """ 用例备注信息 """

    STATE = 'state'
    """ 用例状态 """

    EXECUTE_DATE = 'execute_date'
    """ 执行日期 """

    START_TIME = 'start_time'
    """ 开始时间 """

    END_TIME = 'end_time'
    """ 结束时间 """

    SPEND_TIME_S = 'spend_time_s'
    """ 消耗时间(秒) """

    RUN_STEP = 'run_step'
    """ 运行步骤 """



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from rains.api.common import *
from rains.common.db.const import ConstDatabaseTaskNaming
from rains.common.db.const import ConstDatabaseCaseNaming


# 数据接口蓝图
data_blueprint = Blueprint('data', __name__)


@data_blueprint.route('/data/summarize', methods=['GET'])
def summarize() -> Response:
    """
    [ 获取执行信息概述 ]
    * 无

    [ 必要参数 ]
    * 无

    [ 可选参数 ]
    * 无

    [ 返回内容 ]
    * (Response) : 执行信息概述

    """

    try:
        # 获取最新执行数据
        new_exec_data = {}
        # 获取日期
        date = DB.read(SQL.task.get_date_list())[0][0]
        new_exec_data.update({'date': date})
        # 获取任务数量
        new_exec_data.update({'task_count': DB.read(SQL.task.get_count_from_data({
            ConstDatabaseTaskNaming.EXECUTE_DATE: date}))[0][0]})
        # 获取用例数量
        new_exec_data.update({'case_count': DB.read(SQL.case.get_count_from_data({
            ConstDatabaseCaseNaming.EXECUTE_DATE: date}))[0][0]})
        # 获取异常用例数量
        new_exec_data.update({'fail_case_count': DB.read(SQL.case.get_count_fail_from_data({
            ConstDatabaseTaskNaming.EXECUTE_DATE: date}))[0][0]})
        # 获取消耗时间
        new_exec_data.update({'spend_time': round((DB.read(SQL.task.get_spend_time_from_data({
            ConstDatabaseTaskNaming.EXECUTE_DATE: date}))[0][0] / 60), 2)})

        # 获取历史执行数据
        history_exec_data = {}
        # 获取任务数量
        history_exec_data.update({'task_count': DB.read(SQL.task.get_count_all())[0][0]})
        # 获取用例数量
        history_exec_data.update({'case_count': DB.read(SQL.case.get_count_all())[0][0]})
        # 获取异常用例数量
        history_exec_data.update({'fail_case_count': DB.read(SQL.case.get_count_fail())[0][0]})
        # 获取消耗时间
        history_exec_data.update({'spend_time': round((DB.read(SQL.task.get_spend_time_all())[0][0] / 60), 2)})
        # 获取异常任务数量
        history_exec_data.update({'fail_task_count': DB.read(SQL.task.get_count_fail())[0][0]})

        return ServerParameter.successful({
            'new_exec': new_exec_data,
            'history_exec': history_exec_data
        })

    except BaseException as e:
        return ServerParameter.unsuccessful(f'{ e }')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""

__all__ = ['Log']


import time
import logging

from rains.common.const import ConstPath
from rains.common.decorator import singleton_pattern


# 日志服务::记录器日志级别
_CONFIG_LOGGING_LEVEL = 'DEBUG'

# 日志服务::默认流处理器的日志级别
_CONFIG_BASE_HANDLE_LEVEL = 'INFO'

# 日志服务::文件流处理器的日志级别
_CONFIG_FILE_HANDLE_LEVELS = ['DEBUG', 'INFO', 'ERROR', 'WARNING']

# 日志服务::日志输出格式对象
_CONFIG_OUTPUT_STRUCTURE = logging.Formatter('[%(asctime)s] [%(levelname)7s] %(message)s')


@singleton_pattern
class Log(object):
    """
    [ 日志服务 ]
    * 无

    """

    _logger: logging.Logger
    """ 记录器 """

    def __init__(self):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        try:
            # 创建记录器
            self._logger = logging.getLogger()
            self._logger.setLevel(_CONFIG_LOGGING_LEVEL)
            self._logger.handlers.clear()

            # 创建默认流处理器
            self._creation_base_handle()
            # 创建文件流处理器
            self._creation_file_handle()

        except BaseException as e:
            raise Exception(f"实例化日志服务异常:: { e } ")

    def debug(self, message):
        """
        [ DEBUG ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._logger.debug(message)

    def info(self, message):
        """
        [ INFO ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._logger.info(message)

    def warning(self, message):
        """
        [ WARNING ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._logger.warning(message)

    def error(self, message):
        """
        [ ERROR ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._logger.error(message)

    def critical(self, message):
        """
        [ CRITICAL ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._logger.critical(message)

    def _creation_base_handle(self):
        """
        [ 创建默认流处理器 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        # 创建处理器
        base_handle = logging.StreamHandler()
        # 配置日志输出格式
        base_handle.setFormatter(_CONFIG_OUTPUT_STRUCTURE)
        # 设置处理器日志等级
        base_handle.setLevel(_CONFIG_BASE_HANDLE_LEVEL)
        # 注册处理器
        self._logger.addHandler(base_handle)

    def _creation_file_handle(self):
        """
        [ 创建文件流处理器 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        # 创建当前项目的日志根目录
        if not ConstPath.LOGS.is_dir():
            ConstPath.LOGS.mkdir()

        # 创建当前项目的当天日志存放目录
        date = time.strftime('%Y-%m-%d', time.localtime())
        path = ConstPath.LOGS.joinpath(date)
        if not path.is_dir():
            path.mkdir()

        # 创建文件处理器
        for v in _CONFIG_FILE_HANDLE_LEVELS:

            # 创建处理器
            file_handle = logging.FileHandler(f'{path.joinpath(f"{ v }.log")}')
            # 配置日志输出格式
            file_handle.setFormatter(_CONFIG_OUTPUT_STRUCTURE)
            # 设置处理器的日志等级
            file_handle.setLevel(v)
            # 注册处理器
            self._logger.addHandler(file_handle)



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = ['RunPool']


import gc
import psutil

from multiprocessing.dummy import Pool
from multiprocessing.dummy import Manager

from rains.common.log import Log


class RunPool(object):
    """
    [ 运行池 ]
    * 无

    """

    _log = Log()
    """ 日志服务 """

    _pool: Pool
    """ 进程池 """

    _core_queue: Manager().Queue
    """ 核心队列 """

    _task_queue: Manager().Queue
    """ 任务队列 """

    _pool_state: bool
    """ 进程池运作状态 """

    _sign_task_count: int
    """ 注册的任务数量 """

    _max_quantity_pool: int = 3
    """ 进程最大阈值 """

    _max_quantity_core: int = 5
    """ 核心最大阈值 """

    _max_quantity_task: int = 20
    """ 任务最大阈值 """

    _lock_from_core_queue: Manager().Lock
    """ 核心队列互斥锁 """

    _lock_from_task_queue: Manager().Lock
    """ 任务队列互斥锁 """

    _lock_from_pool_state: Manager().Lock
    """ 进程池运作状态互斥锁 """

    _lock_from_task_count: Manager().Lock
    """ 注册的任务数量互斥锁 """

    def __init__(self, cores: list, tasks: list):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * cores (list) : 核心列表
        * tasks (list) : 任务列表

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        # 初始化赋值
        self._initialization()

        # 遍历核心列表
        for core in cores:
            self._core_queue.put(core)

        # 遍历任务列表
        for task in tasks:
            self._task_queue.put(task)
            self._sign_task_count += 1

    def _initialization(self):
        """
        [ 初始化进程池 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        mgr = Manager()
        self._sign_task_count = 0

        # 创建队列
        self._core_queue = mgr.Queue(maxsize=self._max_quantity_core)
        self._task_queue = mgr.Queue(maxsize=self._max_quantity_task)

        # 创建互斥锁
        self._lock_from_core_queue = mgr.Lock()
        self._lock_from_task_queue = mgr.Lock()
        self._lock_from_pool_state = mgr.Lock()
        self._lock_from_task_count = mgr.Lock()

        # 创建进程池
        self._pool = Pool(self._max_quantity_pool)

    def running(self):
        """
        [ 运行 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        # 获取注册的任务数量
        with self._lock_from_task_count:
            count = self._sign_task_count

        # 存在任务则执行循环
        if count > 0:

            self._pool_state = True

            for i in range(count):
                # 强制 GC 回收垃圾避免内存溢出
                gc.collect()

                # 驱动冷启动时会占用大量的 CPU 资源，为避免因为 CPU 占用率过高导致进程启动异常
                # 每隔 2 秒检查 CPU 占用率，只有 CPU 占用率少于 50 时才会启动进程
                while True:
                    if psutil.cpu_percent(2) < 50:
                        self._pool.apply_async(func=self._emit)
                        break

            # 轮询进程执行情况，当所有进程都执行完毕后，退出程序
            while True:
                if self._pool_state and self._sign_task_count == 0:
                    self._pool_state = False
                    break

        gc.collect()

    def _emit(self):
        """
        [ 触发工作流 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        # 取出队列中的核心与任务
        task = self._task_queue.get()
        core = self._core_queue.get()

        # 执行核心的 running 函数
        r = core.start_task(task)

        # 触发回调
        self._function_call_back()

        # 核心执行完任务后，放回核心队列等待
        with self._lock_from_core_queue:
            self._core_queue.put(core)

        return r

    def _function_call_back(self):
        """
        [ 触发回调 ]
        * 该回调将消耗任务数量。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        with self._lock_from_task_count:
            self._sign_task_count -= 1



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = ['MiniPlantView']


from rains.common.log import Log

from rains.kit.mini.mini_engine import MiniEngine


class MiniPlantView(object):
    """
    [ 小程序视图 ]
    * 提供针对小程序本身的操作函数。

    """

    _log = Log()
    """ [ 日志服务 ] """

    _mini_engine: MiniEngine
    """ [ 引擎对象 ] """

    def __init__(self, mini_engine: MiniEngine):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * mini_engine (MiniEngine) : 小程序引擎对象

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._mini_engine = mini_engine

    def _log_output(self, message: str, log: bool):
        """
        [ 日志输出 ]
        * 无

        [ 必要参数 ]
        * message (str) : 输出信息

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if log:
            self._log.info(f'执行核心[{ self._mini_engine.id }]::{ message }')

    def _err_output(self, message: str):
        """
        [ 错误输出 ]
        * 无

        [ 必要参数 ]
        * message (str) : 输出信息

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        raise Exception(f'执行核心[{ self._mini_engine.id }]::{ message }')

    @property
    def get(self):
        """
        [ WEB视图'获取'相关函数 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        return _MiniPlantViewMapGet(self._mini_engine, self._log_output, self._err_output)

    @property
    def page(self):
        """
        [ WEB视图'获取'相关函数 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        return _MiniPlantViewMapPage(self._mini_engine, self._log_output, self._err_output)

    def save_png(self, save_path: str, log: bool = True):
        """
        [ 保存快照 ]
        * ide上仅能截取到 wxml 页面的内容，Modal/Actionsheet/授权弹窗等无法截取。

        [ 必要参数 ]
        * save_path (str) : 保存 .png 的路径

        [ 可选参数 ]
        * log (bool) : 日志开关

        [ 返回内容 ]
        * 无

        """

        try:
            self._mini_engine.driver.app.screen_shot(str(save_path))
            self._log_output(f'保存快照[{ save_path }]', log)

            return self

        except BaseException as e:
            self._err_output(f'保存快照异常::{ e }')


class _MiniPlantViewMapBase(object):
    """
    [ 小程序视图封装基类 ]
    * 无

    """

    _mini_engine: MiniEngine
    """ [ 引擎对象 ] """

    _log_output = None
    """ [ 日志输出函数 ] """

    _err_output = None
    """ [ 错误输出函数 ] """

    def __init__(self, mini_engine: MiniEngine, log_output_function, err_output_function):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * mini_engine (MiniEngine) : 小程序引擎对象
        * log_output_function (function) : 日志输出函数
        * err_output_function (function) : 错误输出函数

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._mini_engine = mini_engine
        self._log_output = log_output_function
        self._err_output = err_output_function


class _MiniPlantViewMapGet(_MiniPlantViewMapBase):
    """
    [ 小程序视图'获取'相关函数 ]
    * 无

    """

    def data(self, log: bool = True) -> str:
        """
        [ 获取当前页面数据 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * log (bool) : 日志开关

        [ 返回内容 ]
        * (str) : 当前页面数据

        """

        try:
            data = self._mini_engine.driver.app.get_current_page().data
            self._log_output('获取当前页面数据', log)

            return data

        except BaseException as e:
            self._err_output(f'获取当前页面数据异常:: { e }')

    # func boundary -----------------------------------------------------------------------------------------------

    def pages(self, log: bool = True) -> str:
        """
        [ 获取所有已配置的页面路径 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * log (bool) : 日志开关

        [ 返回内容 ]
        * (str) : 当前页面数据

        """

        try:
            paths = self._mini_engine.driver.app.get_all_pages_path()
            self._log_output('获取所有已配置的页面路径', log)

            return paths

        except BaseException as e:
            self._err_output(f'获取所有已配置的页面路径异常:: { e }')

    # func boundary -----------------------------------------------------------------------------------------------

    def current_page(self, log: bool = True) -> str:
        """
        [ 获取当前顶层页面 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * log (bool) : 日志开关

        [ 返回内容 ]
        * (str) : 当前页面数据

        """

        try:
            page = self._mini_engine.driver.app.get_current_page()
            path = page.path

            self._log_output(f'获取当前顶层页面[{ path }]', log)

            return path

        except BaseException as e:
            self._err_output(f'获取当前顶层页面异常:: { e }')


class _MiniPlantViewMapPage(_MiniPlantViewMapBase):
    """
    [ 小程序视图'页面'相关函数 ]
    * 无

    """

    def to_home(self, log: bool = True):
        """
        [ 跳转至小程序首页 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * log (bool) : 日志开关

        [ 返回内容 ]
        * (str) : 当前页面数据

        """

        try:
            self._mini_engine.driver.app.go_home()
            self._log_output('跳转至小程序首页', log)

            return self

        except BaseException as e:
            self._err_output(f'跳转至小程序首页异常:: { e }')

    def to_page(self, page_path: str, params: dict = None, is_wait: bool = True, log: bool = True):
        """
        [ 跳转页面 ]
        * 以导航的方式跳转到指定页面。
        * 不能跳到 tabBar 页面，支持相对路径和绝对路径，小程序中页面栈最多十层。
        * 页面路径规则：
        * /page/tabBar/API/index: 绝对路径,最前面为/。
        * tabBar/API/index: 相对路径, 会被拼接在当前页面父节点的路径后面。

        [ 必要参数 ]
        * page_path (str) : 页面路径

        [ 可选参数 ]
        * params (dict) : 页面参数
        * is_wait (bool) : 是否等待新的页面跳转
        * log (bool) : 日志开关

        [ 返回内容 ]
        * 无

        """

        try:
            self._mini_engine.driver.app.navigate_to(page_path, params, is_wait)
            self._log_output(f'跳转至页面[{ page_path }]', log)

            return self

        except BaseException as e:
            self._err_output(f'跳转至页面异常:: { e }')

    def back(self, delta: int = 1, log: bool = True):
        """
        [ 返回页面 ]
        * 关闭当前页面，返回上一页面或多级页面。
        * 如果超出当前页面栈最大层数，返回首页。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * delta (int) : 返回的层数
        * log (bool) : 日志开关

        [ 返回内容 ]
        * 无

        """

        try:
            self._mini_engine.driver.app.navigate_back(delta)
            self._log_output(f'返回上[{ delta }]层页面，', log)

            return self

        except BaseException as e:
            self._err_output(f'返回页面异常:: { e }')

    def switch_tab(self, tab_path: int = 1, log: bool = True):
        """
        [ 跳转至 tabBar 页面 ]
        * 会关闭其他所有非 tabBar 页面。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * tab_path (int) : 需要跳转的 tabBar 页面的路径（需在 app.json 的 tabBar 字段定义的页面），路径后不能带参数
        * log (bool) : 日志开关

        [ 返回内容 ]
        * 无

        """

        try:
            self._mini_engine.driver.app.switch_tab(tab_path)
            self._log_output(f'跳转至 tabBar 页面[{ tab_path }]，', log)

            return self

        except BaseException as e:
            self._err_output(f'跳转至 tabBar 页面异常:: { e }')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from selenium import webdriver
from selenium.webdriver.chrome.webdriver import RemoteWebDriver

from rains.common.log import Log
from rains.common.i.i_engine import IEngine
from rains.common.const import ConstProjectType

from rains.kit.web.packaging.web_find_handle import WebFindHandle
from rains.kit.web.web_const import BrowserType


class WebEngine(IEngine):
    """
    [ WEB引擎 ]
    * 引擎封装了原生驱动的获取逻辑与一个基于原生 Selenium3 二次实现的元素定位器，并且管理着驱动对象的生命周期。

    """

    _log = Log()
    """ WEB引擎 """

    _state: bool
    """ 状态 """

    _core_id: int
    """ 编号 """

    _type: str = ConstProjectType.WEB
    """ 类型 """

    _browser_type: str = BrowserType.CHROME
    """ 浏览器类型 """

    _driver: RemoteWebDriver = None
    """ 原生驱动对象 """

    _find_handle: WebFindHandle = None
    """ 元素定位器 """

    def __init__(self, core_id: int):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * core_id (int): 核心ID

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._core_id = core_id
        self._state = False

    @property
    def core_id(self):
        return self._core_id

    @property
    def type(self):
        return self._type

    @property
    def browser_type(self):
        return self.browser_type

    @property
    def driver(self):
        return self._driver

    @property
    def find_handle(self):
        return self._find_handle

    def get_state(self) -> bool:
        """
        [ 获取引擎状态 ]
        * 获取当前引擎状态，启动中返回 True，否则返回 False。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (Bool) : 当前引擎状态

        """

        return self._state

    def set_browser_type(self, browser_type: str):
        """
        [ 设置浏览器类型 ]
        * 无

        [ 必要参数 ]
        * browser_type (str) : 浏览器类型

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._browser_type = browser_type
        return self

    def run(self):
        """
        [ 运行引擎 ]
        * 驱动与定位器的实例化。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if not self._state:

            # 谷歌浏览器
            if self._browser_type == BrowserType.CHROME:
                # 以非 W3C 模式运行
                # 不显示自动化劫持 && 不打印日志
                # 创建浏览器驱动实例
                opt = webdriver.ChromeOptions()
                opt.add_experimental_option('w3c', False)
                opt.add_experimental_option('excludeSwitches', ['enable-automation', 'enable-logging'])
                self._driver = webdriver.Chrome(options=opt)
                self._find_handle = WebFindHandle(self._driver)
                self._state = True

        return self

    def end(self):
        """
        [ 结束引擎 ]
        * 退出并销毁当前持有的驱动与定位器。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if self._state:

            self._driver.quit()
            self._state = False

            del self._driver
            del self._find_handle

            return self

    def reset(self):
        """
        [ 重置引擎 ]
        * 将引擎重置至初始阶段。
        * 与 end() 相比，该函数只是重置引擎至初始状态，而不是注销。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if self._state:

            # 打开一个新空白标签页
            self._driver.execute_script(f'window.open("")')

            # 关闭所有其他标签页面
            new_page_handle = self._driver.window_handles[-1]

            for page_handle in self._driver.window_handles:
                if page_handle != new_page_handle:
                    self._driver.switch_to.window(page_handle)
                    self._driver.close()

            # 句柄回到新空白标签页
            self._driver.switch_to.window(new_page_handle)

        return self



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = ['SqlTask']


from rains.common.db.sql_packaging.common import *


class SqlTask(object):
    """
    [ 任务相关的 SQL 语句 ]
    * 无

    """

    @staticmethod
    def get_count_all() -> str:
        """
        [ 查询所有任务数量 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        return f""" 
        
        SELECT COUNT(id)
        
        FROM { ConstDatabaseTableNaming.TASKS }
        
        """

    @staticmethod
    def get_count_pass() -> str:
        """
        [ 查询全部通过的任务数量 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        return f""" 
        
        SELECT COUNT(id)

        FROM { ConstDatabaseTableNaming.TASKS }

        WHERE { ConstDatabaseTaskNaming.CASE_FAIL } == 0

        """

    @staticmethod
    def get_count_fail() -> str:
        """
        [ 查询存在异常的任务数量 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        return f""" 

        SELECT COUNT(id)

        FROM { ConstDatabaseTableNaming.TASKS }

        WHERE { ConstDatabaseTaskNaming.CASE_FAIL } != 0

        """

    @staticmethod
    def get_count_from_data(paras: dict = None) -> str:
        """
        [ 查询指定日期中的任务数量 ]
        * 无

        [ 必要参数 ]
        * execute (str) : 执行日期

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseTaskNaming.EXECUTE_DATE
            ]
        )

        return f""" 

        SELECT COUNT(id)

        FROM { ConstDatabaseTableNaming.TASKS }

        WHERE { ConstDatabaseTaskNaming.EXECUTE_DATE } = '{ paras[ConstDatabaseTaskNaming.EXECUTE_DATE] }'

        """

    @staticmethod
    def get_date_list() -> str:
        """
        [ 查询去重后的所有存在任务的日期列表 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        return f"""

        SELECT DISTINCT { ConstDatabaseTaskNaming.EXECUTE_DATE } 

        FROM { ConstDatabaseTableNaming.TASKS }

        ORDER BY ID DESC

        """

    @staticmethod
    def get_spend_time_all() -> str:
        """
        [ 查询所有任务的消耗时间 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        return f""" 

        SELECT SUM({ ConstDatabaseTaskNaming.SPEND_TIME_S })

        FROM { ConstDatabaseTableNaming.TASKS }

        """

    @staticmethod
    def get_spend_time_from_data(paras: dict = None) -> str:
        """
        [ 查询指定日期中所有任务的消耗时间 ]
        * 无

        [ 必要参数 ]
        * execute (str) : 执行日期

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseTaskNaming.EXECUTE_DATE
            ]
        )

        return f""" 

        SELECT SUM({ ConstDatabaseTaskNaming.SPEND_TIME_S })

        FROM { ConstDatabaseTableNaming.TASKS }

        WHERE { ConstDatabaseTaskNaming.EXECUTE_DATE } = '{ paras[ConstDatabaseTaskNaming.EXECUTE_DATE] }'

        """

    @staticmethod
    def get_info_all(paras: dict) -> str:
        """
        [ 查询所有任务详情信息 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * page (int): 页数
        * number (int): 查询个数

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        return f"""

        SELECT * 

        FROM { ConstDatabaseTableNaming.TASKS }

        ORDER BY ID DESC 

        { DatabaseParameter.get_desc_limit(paras) }

        """

    @staticmethod
    def get_info_from_tid(paras: dict = None) -> str:
        """
        [ 查询指定TID的任务详情信息 ]
        * 无

        [ 必要参数 ]
        * tid (int) : 任务ID

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseCaseNaming.TID
            ]
        )

        return f"""

        SELECT *

        FROM { ConstDatabaseTableNaming.TASKS }

        WHERE ID = { paras[ConstDatabaseCaseNaming.TID] }

        """

    @staticmethod
    def add(paras: dict = None) -> str:
        """
        [ 创建任务 ]
        * 无

        [ 必要参数 ]
        * name (str) : 任务名称
        * remark (str) : 任务备注
        * execute_date (date) : 执行日期

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseTaskNaming.NAME,
                ConstDatabaseTaskNaming.REMARK,
                ConstDatabaseTaskNaming.EXECUTE_DATE
            ]
        )

        return f""" 

        INSERT INTO { ConstDatabaseTableNaming.TASKS } (

             id, 
             { ConstDatabaseTaskNaming.NAME },
             { ConstDatabaseTaskNaming.REMARK },
             { ConstDatabaseTaskNaming.EXECUTE_DATE },
             { ConstDatabaseTaskNaming.IS_COMPLETED },
             { ConstDatabaseTaskNaming.START_TIME },
             { ConstDatabaseTaskNaming.END_TIME },
             { ConstDatabaseTaskNaming.SPEND_TIME_S },
             { ConstDatabaseTaskNaming.CASE_ALL },
             { ConstDatabaseTaskNaming.CASE_PASS },
             { ConstDatabaseTaskNaming.CASE_FAIL }
        )

        VALUES (

            NULL,
            '{ paras[ConstDatabaseTaskNaming.NAME] }',
            '{ paras[ConstDatabaseTaskNaming.REMARK] }',
            '{ paras[ConstDatabaseTaskNaming.EXECUTE_DATE] }',
             0, NULL, NULL, NULL, NULL, NULL, NULL
        )

        """

    @staticmethod
    def update(paras: dict = None) -> str:
        """
        [ 更新任务 ]
        * 无

        [ 必要参数 ]
        * tid (int) : 所属任务ID
        * is_completed (bool) : 是否已完成
        * start_time (date) : 开始时间
        * end_time (date) : 结束时间
        * spend_time_s (int) : 花费时间(秒)
        * case_all (int) : 所有用例数
        * case_pass (int) : 成功的用例数
        * case_fail (int) : 失败用例数量

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseTaskNaming.TID,
                ConstDatabaseTaskNaming.IS_COMPLETED,
                ConstDatabaseTaskNaming.START_TIME,
                ConstDatabaseTaskNaming.END_TIME,
                ConstDatabaseTaskNaming.SPEND_TIME_S,
                ConstDatabaseTaskNaming.CASE_ALL,
                ConstDatabaseTaskNaming.CASE_PASS,
                ConstDatabaseTaskNaming.CASE_FAIL
            ]
        )

        return f"""

        UPDATE { ConstDatabaseTableNaming.TASKS }

        SET
            { ConstDatabaseTaskNaming.IS_COMPLETED }  = { paras[ConstDatabaseTaskNaming.IS_COMPLETED] },
            { ConstDatabaseTaskNaming.START_TIME }    = '{ paras[ConstDatabaseTaskNaming.START_TIME] }',
            { ConstDatabaseTaskNaming.END_TIME }      = '{ paras[ConstDatabaseTaskNaming.END_TIME] }',
            { ConstDatabaseTaskNaming.SPEND_TIME_S }  = '{ paras[ConstDatabaseTaskNaming.SPEND_TIME_S] }',
            { ConstDatabaseTaskNaming.CASE_ALL }      =  { paras[ConstDatabaseTaskNaming.CASE_ALL] },
            { ConstDatabaseTaskNaming.CASE_PASS }     =  { paras[ConstDatabaseTaskNaming.CASE_PASS] },
            { ConstDatabaseTaskNaming.CASE_FAIL }     =  { paras[ConstDatabaseTaskNaming.CASE_FAIL] }

        WHERE ID = { paras[ConstDatabaseTaskNaming.TID] }

        """

    @staticmethod
    def delete(paras: dict) -> str:
        """
        [ 删除任务 ]
        * 无

        [ 必要参数 ]
        * tid (int) : 所属任务ID

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        DatabaseParameter.machining_parameter(
            paras,
            [
                ConstDatabaseTaskNaming.TID
            ]
        )

        return f"""

        DELETE FROM { ConstDatabaseTableNaming.TASKS }

        WHERE ID = { paras[ConstDatabaseTaskNaming.TID] }

        """

    @staticmethod
    def delete_all() -> str:
        """
        [ 删除所有的任务 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接完的 SQL 语句

        """

        return f"""

        DELETE FROM { ConstDatabaseTableNaming.TASKS }

        """



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


from rains.api.common import *


# 用例接口蓝图
case_blueprint = Blueprint('case', __name__)


@case_blueprint.route('/case/cases', methods=['POST'])
def cases() -> Response:
    """
    [ 获取用例信息列表 ]
    * 无

    [ 必要参数 ]
    * tid (int) : 所属任务ID

    [ 可选参数 ]
    * state (str) : 状态
    * page (int) : 页数
    * number (int) : 查询数量

    [ 返回内容 ]
    * (Response) : 指定 tid 任务的所有用例信息列表

    """

    try:
        # 获取请求参数
        api_paras = ServerParameter.analysis_request_paras('tid', 'state', 'page', 'number')
        # 获取服务器数据
        base_case_list = DB.read(SQL.case.get_info_from_tid(api_paras))

        # 加工数据
        # 将用例信息中的运行步骤转换成列表然后重新拼接数据
        new_case_list = []
        for case_info in base_case_list:
            new_case_info = []
            number = 0
            for info in case_info:
                if number == 9:
                    info = info.split('\n')
                new_case_info.append(info)
                number += 1
            new_case_list.append(new_case_info)
        return ServerParameter.successful({'cases': new_case_list})

    except BaseException as e:
        return ServerParameter.unsuccessful(f'{ e }')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


import requests

from flask import Flask
from flask import request
from flask import render_template

from rains.api.blueprint.data import data_blueprint
from rains.api.blueprint.task import task_blueprint
from rains.api.blueprint.case import case_blueprint


# 注册蓝图
app = Flask(__name__)
app.register_blueprint(data_blueprint)
app.register_blueprint(task_blueprint)
app.register_blueprint(case_blueprint)

# 配置项
app.config.update({'JSON_AS_ASCII': False})

# 声明静态文件路径
app._static_folder = './static'


@app.route('/', methods=['GET'])
@app.route('/index', methods=['GET'])
def index():
    """
    [ 概述 ]
    * 无

    [ 必要参数 ]
    * 无

    [ 可选参数 ]
    * 无

    [ 返回内容 ]
    * index.html / 500.html

    """

    r = requests.get(''.join([request.host_url, 'data/summarize'])).json()
    return _return_render_page(r, 'index.html')


@app.route('/task/<page>', methods=['GET'])
def task(page):
    """
    [ 任务 ]
    * 无

    [ 必要参数 ]
    * 无

    [ 可选参数 ]
    * 无

    [ 返回内容 ]
    * task.html / 500.html

    """

    r = requests.post(''.join([request.host_url, 'task/tasks']), data={'page': page}).json()
    return _return_render_page(r, 'task.html')


@app.route('/case/<tid>', methods=['GET'])
def case(tid):
    """
    [ 用例 ]
    * 无

    [ 必要参数 ]
    * 无

    [ 可选参数 ]
    * 无

    [ 返回内容 ]
    * case.html / 500.html

    """

    r = requests.post(''.join([request.host_url, 'case/cases']), data={'tid': tid}).json()
    return _return_render_page(r, 'case.html')


@app.errorhandler(404)
def error_404(error):
    """
    [ 404 ]
    * 无

    [ 必要参数 ]
    * 无

    [ 可选参数 ]
    * 无

    [ 返回内容 ]
    * 404.html

    """

    print(error)
    return render_template('404.html'), 404


@app.errorhandler(500)
def error_500(error):
    """
    [ 500 ]
    * 无

    [ 必要参数 ]
    * 无

    [ 可选参数 ]
    * 无

    [ 返回内容 ]
    * 500.html

    """

    print(error)
    return render_template('500.html'), 500


def _return_render_page(return_data: dict, html_name: str):
    """
    [ 渲染页面 ]
    * 根据接口返回页面。

    [ 必要参数 ]
    * 无

    [ 可选参数 ]
    * 无

    [ 返回内容 ]
    * {html_name}.html / 500.html

    """

    if return_data['code'] == 200:
        return render_template(html_name, Data=return_data['data'])
    else:
        return render_template('500.html'), 500



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = [
    'DB',
    'SQL',
    'Response',
    'Blueprint',
    'ServerParameter',
]


from flask import jsonify
from flask import request
from flask import Response
from flask import Blueprint

from rains.common.db.sql_lang import SqlLang
from rains.common.db.database import Database


# 通用实例
DB: Database = Database()
SQL: SqlLang = SqlLang()


class ServerParameter(object):
    """
    [ 服务端参数类 ]
    * 无

    """

    @staticmethod
    def analysis_request_paras(*paras) -> dict:
        """
        [ 解析请求参数 ]
        * 无

        [ 必要参数 ]
        * *paras (str) : 需要解析的参数 Key, 允许传递多个值

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (dict) : 返回解析成功的参数字典

        """

        try:
            # 解析请求模式
            request_handle = request.args
            if request.method == 'POST':
                request_handle = request.form

            # 获取参数并且返回
            decryption_paras = {}
            for key in paras:
                value = request_handle.get(key)
                if value is None:
                    pass
                else:
                    decryption_paras.update({key: value})
            return decryption_paras

        except BaseException as e:
            raise Exception(f"解析请求接口携带的参数异常:: { e }")

    @staticmethod
    def successful(paras: dict or None) -> jsonify:
        """
        [ 请求成功 ]
        * 无

        [ 必要参数 ]
        * paras （dict | none） : 需要返回的数据字典, 默认 None 则返回空字典

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (Json) : 经过加工的 Json 数据结构

        """

        r_structure = {
            'code': 200,
            'message': '成功',
            'data': {}
        }

        if paras:
            r_structure.update({'data': paras})

        return jsonify(r_structure)

    @staticmethod
    def unsuccessful(err: str) -> jsonify:
        """
        [ 请求失败 ]
        * 无

        [ 必要参数 ]
        * err (str) : 需要返回的错误信息

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (Json) : 经过加工的 Json 数据结构

        """

        r_structure = {
            'code': 500,
            'message': '失败',
            'err': err
        }

        return jsonify(r_structure)



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


__all__ = [
    'DatabaseParameter',
    'ConstTaskAndCaseState',
    'ConstDatabaseTaskNaming',
    'ConstDatabaseCaseNaming',
    'ConstDatabaseTableNaming',
]


from rains.common.const import ConstTaskAndCaseState
from rains.common.db.const import ConstDatabaseTaskNaming
from rains.common.db.const import ConstDatabaseCaseNaming
from rains.common.db.const import ConstDatabaseTableNaming


class DatabaseParameter(object):
    """
    [ 数据库参数类 ]
    * 无

    """

    @staticmethod
    def get_desc_limit(paras: dict or None) -> str:
        """
        [ 获取数据返回量区间限制 ]
        * 默认是获取 1 页， 10 条数据。

        [ 必要参数 ]
        * paras (dict | none) : 参数字典

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * (str) : 拼接好的 SQL LIMIT 语句

        """

        # 获取参数
        paras = {} if not paras else paras
        page = 1 if 'page' not in paras.keys() else int(paras['page'])
        number = 10 if 'number' not in paras.keys() else int(paras['number'])

        # 计算偏移量
        limit_begin = 0
        if page > 1:
            limit_begin = (page - 1) * number

        return f'LIMIT { limit_begin }, { number }'

    @staticmethod
    def machining_parameter(base_paras: dict, must_keys: list):
        """
        [ 检查参数是否包含必要 key ]
        * 如果 must_keys 的所有元素都存在于 base_paras 中，则通过，否则抛出 ParametersAreMissingException 异常。

        [ 必要参数 ]
        * base_paras (dict) : 参数字典
        * must_keys (list) : 必要 key 列表

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        base_paras = {} if not base_paras else base_paras

        for key in must_keys:
            try:
                if base_paras[key] is None:
                    pass
            except KeyError:
                raise ParametersAreMissingException(key)


class ParametersAreMissingException(Exception):
    """
    [ 参数缺失错误 ]
    * 无

    """

    def __init__(self, missing_para_key):
        Exception.__init__(self, f'数据库必要参数 { missing_para_key } 缺失!')



# !/usr/bin/env python
# coding:utf-8

"""
Copyright (c) 2021. quinn.7@foxmail.com All rights reserved.
Based on the Apache License 2.0 open source protocol.

作者 cat7
邮箱 quinn.7@foxmail.com

"""


import time

from selenium.webdriver import ActionChains
from selenium.webdriver.support.select import Select

from rains.common.log import Log
from rains.common.task.case_recorder import CaseRecorder

from rains.kit.web.web_engine import WebEngine
from rains.kit.web.packaging.web_element_structure import WebElementStructure


class WebPlantElement(object):
    """
    [ WEB元素控件 ]
    * 基于元素特征构建指向该元素的元素控件对象。

    """

    _log = Log()
    """ 日志服务 """

    _web_engine: WebEngine
    """ 引擎对象 """

    _case_recorder: CaseRecorder = CaseRecorder()
    """ 用例记录器 """

    _element_structure: WebElementStructure
    """ 元素结构体 """

    _element_container: list = []
    """ 元素列表容器 """

    def __init__(self, engine: WebEngine, element_structure: WebElementStructure):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * engine (WebEngine) : WEB引擎对象
        * element_structure (WebElementStructure) : WEB元素结构体

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._web_engine = engine
        self._element_structure = element_structure

    @property
    def get(self):
        """
        [ WEB元素控件'获取'相关函数 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        return _WebPlantElementMapGet(self._web_engine, self._analytical_elements, self._log_out, self._err_out)

    @property
    def mouse(self):
        """
        [ WEB元素控件'鼠标'相关函数 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        return _WebPlantElementMapMouse(self._web_engine, self._analytical_elements, self._log_out, self._err_out)

    @property
    def input(self):
        """
        [ WEB元素控件'输入框'相关函数 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        return _WebPlantElementMapInput(self._web_engine, self._analytical_elements, self._log_out, self._err_out)

    @property
    def selector(self):
        """
        [ WEB元素控件'下拉框'相关函数 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        return _WebPlantElementMapSelector(self._web_engine, self._analytical_elements, self._log_out, self._err_out)

    def _log_out(self, message: str, log: bool):
        """
        [ 日志输出 ]
        * 无

        [ 必要参数 ]
        * message (str) : 输出信息

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        if log:
            message = f'执行核心[{ self._web_engine.core_id }]::' \
                      f'{ self._element_structure.page }::' \
                      f'{ self._element_structure.name }::' \
                      f'{ message }'

            self._log.info(message)
            self._case_recorder.write(self._web_engine.core_id, message)

            return message

    def _err_out(self, message: str):
        """
        [ 错误输出 ]
        * 无

        [ 必要参数 ]
        * message (str) : 输出信息

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        message = self._log_out(message, True)
        raise Exception(message)

    def _analytical_elements(self, wait: int = 15, tolerance: bool = False) -> list:
        """
        [ 解析函数 ]
        * 每一次调用页面元素组件的方法时，都会调用该函数，以更新定位结果，确认组件是否存在于当前页面中。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * wait (int) : 查找元素所等待的显式等待时长
        * tolerance (bool) : 容错模式，为 True 则表示定位不到元素也不报错，返回空列表

        [ 返回内容 ]
        * 元素列表容器

        """

        self._element_container = \
            self._web_engine.find_handle.find(self._element_structure, wait, tolerance)

        return self._element_container


class _WebPlantElementMapBase(object):
    """
    [ WEB元素控件封装基类 ]
    * NULL

    """

    _web_engine: WebEngine
    """ 引擎对象 """

    _element_container: list = []
    """ 元素列表容器 """

    _analytical_elements = None
    """ 解析函数 """

    _log_out = None
    """ 日志输出函数 """

    _err_out = None
    """ 错误输出函数 """

    def __init__(self, web_element: WebEngine, analytical_elements_function, log_output_function, err_output_function):
        """
        [ 初始化 ]
        * 无

        [ 必要参数 ]
        * web_element (WebEngine) : WEB引擎对象
        * analytical_elements_function (function) : 元素解析函数
        * log_output_function (function) : 日志输出函数
        * err_output_function (function) : 错误输出函数

        [ 可选参数 ]
        * 无

        [ 返回内容 ]
        * 无

        """

        self._web_engine = web_element
        self._analytical_elements = analytical_elements_function
        self._log_out = log_output_function
        self._err_out = err_output_function


class _WebPlantElementMapGet(_WebPlantElementMapBase):
    """
    [ WEB元素控件'获取'相关函数 ]
    * NULL

    """

    def count(self, log: bool = True) -> int:
        """
        [ 获取元素数量 ]
        * 定位元素并且返回元素数量。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * log (bool) : 日志开关

        [ 返回内容 ]
        * (int) : 元素数量

        """

        try:
            self._element_container = self._analytical_elements()
            count = len(self._element_container)
            self._log_out(f'获取元素列表数量[{ count }]', log)
            return count

        except BaseException as e:
            self._err_out(f'获取元素列表数量异常:: { e }')

    def text(self, eid: int = 0, log: bool = True) -> str:
        """
        [ 获取文本 ]
        * 返回元素的 text 属性值。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回 ]
        * (str) : 元素文本

        """

        try:
            self._element_container = self._analytical_elements()
            text = self._element_container[eid].text
            self._log_out(f'获取文本[{ text }]', log)
            return text

        except BaseException as e:
            self._err_out(f'获取文本异常:: { e }')

    def value(self, eid: int = 0, log: bool = True) -> str:
        """
        [ 获取 value 值 ]
        * 返回元素的 value 属性值。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回 ]
        * (str) : 元素的值

        """

        try:
            self._element_container = self._analytical_elements()
            value = self._element_container[eid].get_attribute('value')
            self._log_out(f'获取 value 值[{ value }]', log)
            return value

        except BaseException as e:
            self._err_out(f'获取 value 值异常:: { e }')

    def in_page(self, eid: int = 0, wait: int = 3, log: bool = True) -> bool:
        """
        [ 判断元素是否存在当前可视页面 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * wait (int) : 查找元素所等待的显式等待时长
        * log (bool) : 日志开关

        [ 返回参数 ]
        * (bool) : 布尔值

        """

        try:
            self._element_container = self._analytical_elements(wait=wait, tolerance=True)

            proof = False
            if len(self._element_container) >= (eid + 1):
                proof = True
            self._log_out(f'判断元素是否存在当前可视页面[{ proof }]', log)

            return proof

        except BaseException as e:
            self._err_out(f'判断元素是否存在当前可视页面异常:: { e }')


class _WebPlantElementMapMouse(_WebPlantElementMapBase):
    """
    [ WEB元素控件'鼠标'相关函数 ]
    * NULL

    """

    def tap(self, eid: int = 0, log: bool = True):
        """
        [ 左键点击 ]
        * 模拟一次作用于 WebElement 对象上的鼠标左键单击操作。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()
            ActionChains(self._web_engine.driver).click(self._element_container[eid]).perform()
            self._log_out('执行[左键点击]', log)
            return self

        except BaseException as e:
            self._err_out(f'执行[左键点击]异常:: { e }')

    def double_tap(self, eid: int = 0, log: bool = True):
        """
        [ 左键双击 ]
        * 模拟一次作用于 WebElement 对象上的鼠标左键双击操作。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()
            ActionChains(self._web_engine.driver).double_click(self._element_container[eid]).perform()
            self._log_out('执行[左键双击]', log)
            return self

        except BaseException as e:
            self._err_out(f'执行[左键双击]异常:: { e }')

    def long_tap(self, eid: int = 0, sleep: int = 2, log: bool = True):
        """
        [ 左键长按 ]
        * 模拟一次作用于 WebElement 对象上的鼠标左键长按操作。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * sleep (int): 长按时长，默认为 2s
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()
            ActionChains(self._web_engine.driver).click_and_hold(self._element_container[eid]).perform()
            time.sleep(sleep)
            ActionChains(self._web_engine.driver).release(self._element_container[eid]).perform()
            self._log_out('执行[左键长按]', log)
            return self

        except BaseException as e:
            self._err_out(f'执行[左键长按]异常:: { e }')

    def move_to(self, eid: int = 0, log: bool = True):
        """
        [ 鼠标悬停 ]
        * 模拟一次作用于 WebElement 对象上的鼠标悬停操作。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()
            ActionChains(self._web_engine.driver).move_to_element(self._element_container[eid]).perform()
            self._log_out('执行[鼠标悬停]', log)
            return self

        except BaseException as e:
            self._err_out(f'执行[鼠标悬停]异常:: { e }')


class _WebPlantElementMapInput(_WebPlantElementMapBase):
    """
    [ WEB元素控件'输入框'相关函数 ]
    * NULL

    """

    def send(self, key: str, eid: int = 0, log: bool = True):
        """
        [ 输入内容 ]
        * 输入内容，一般指向 input 标签组件。

        [ 必要参数 ]
        * key (str) : 内容文本

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()
            self._element_container[eid].clear()
            self._element_container[eid].send_keys(key)
            self._log_out(f'输入文本[{ key }]', log)
            return self

        except BaseException as e:
            self._err_out(f'输入文本异常:: { e }')

    def clear(self, eid: int = 0, log: bool = True):
        """
        [ 清除内容 ]
        * 清除内容，一般指向 input 标签组件。

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._element_container = self._analytical_elements()
            self._element_container[eid].clear()
            self._log_out(f'清除输入框文本', log)
            return self

        except BaseException as e:
            self._err_out(f'清除输入框文本异常:: { e }')


class _WebPlantElementMapSelector(_WebPlantElementMapBase):
    """
    [ WEB元素控件'下拉框'相关函数 ]
    * NULL

    """

    _select_handle: Select
    """ 下拉框处理器 """

    def __init__(self, web_element: WebEngine, analytical_elements_function, log_output_function, err_output_function):
        """
        [ 初始化 ]
        * NULL

        [ 必要参数 ]
        * web_element (WebEngine) : WEB引擎对象
        * analytical_elements_function (function) : 元素解析函数
        * log_output_function (function) : 日志输出函数
        * err_output_function (function) : 错误输出函数

        [ 可选参数 ]
        * 无

        [ 返回参数 ]
        * 无

        """

        super(_WebPlantElementMapSelector, self).__init__(web_element,
                                                          analytical_elements_function,
                                                          log_output_function,
                                                          err_output_function)

    def get_option_all_item(self, eid: int = 0, log: bool = True) -> list:
        """
        [ 获取下拉框中所有的选项文本 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 包含下拉框中所有的选项文本的列表

        """

        try:
            self._init_select_handle(eid)

            texts = []
            option_list = self._select_handle.options
            for i in option_list:
                texts.append(i.text)

            self._log_out(f'获取所有的选项文本', log)
            return texts

        except BaseException as e:
            self._err_out(f'获取所有的选项文本:: { e }')

    def get_option_all_select_item(self, eid: int = 0, log: bool = True) -> list:
        """
        [ 获取下拉框中所有的已选中选项文本 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 包含下拉框中所有的已选中选项文本的列表

        """

        try:
            self._init_select_handle(eid)

            texts = []
            option_list = self._select_handle.all_selected_options
            for i in option_list:
                texts.append(i.text)

            self._log_out(f'获取所有的已选中选项文本', log)
            return texts

        except BaseException as e:
            self._err_out(f'获取所有的已选中选项文本:: { e }')

    def get_option_current_item(self,
                                eid: int = 0,
                                log: bool = True) -> str:
        """
        [ 获取下拉框中当前选择的首个选项文本 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid : 指定元素下标
        * log : 日志开关

        [ 返回内容 ]
        * 下拉框中当前选择的首个选项文本

        """

        try:
            self._init_select_handle(eid)
            text = self._select_handle.first_selected_option.text
            self._log_out(f'获取当前选择的首个选项文本[{ text }]', log)
            return text

        except BaseException as e:
            self._err_out(f'获取当前选择的首个选项文本:: { e }')

    def select_by_index(self, index: int, eid: int = 0, log: bool = True):
        """
        [ 下拉框中通过下标选择选项 ]
        * 无

        [ 必要参数 ]
        * index (int) : 需要选中的选项下标

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._init_select_handle(eid)
            self._select_handle.select_by_index(index)
            select_option_text = self._select_handle.first_selected_option.text
            self._log_out(f'通过下标选择选项[{ select_option_text }]', log)
            return self

        except BaseException as e:
            self._err_out(f'通过下标选择选项:: { e }')

    def select_by_value(self, value: str, eid: int = 0, log: bool = True):
        """
        [ 下拉框中通过选项标签属性值选择 ]
        * 无

        [ 必要参数 ]
        * value (str) : 需要选中的选项标签属性值

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._init_select_handle(eid)
            self._select_handle.select_by_value(value)
            select_item_text = self._select_handle.first_selected_option.text
            self._log_out(f'通过标签属性值选择选项[{ select_item_text }]', log)
            return self

        except BaseException as e:
            self._err_out(f'通过标签属性值选择选项:: { e }')

    def select_by_text(self, text: str, eid: int = 0, log: bool = True):
        """
        [ 下拉框中通过选项的文本选择 ]
        * 无

        [ 必要参数 ]
        * text (str) : 需要选中的选项文本

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._init_select_handle(eid)
            self._select_handle.select_by_visible_text(text)
            select_item_text = self._select_handle.first_selected_option.text
            self._log_out(f'通过选项的文本选择[{ select_item_text }]', log)
            return self

        except BaseException as e:
            self._err_out(f'通过选项的文本选择:: { e }')

    def deselect_all(self, eid: int = 0, log: bool = True):
        """
        [ 下拉框中取消所有的已选中选项 ]
        * 无

        [ 必要参数 ]
        * 无

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._init_select_handle(eid)
            self._select_handle.deselect_all()
            self._log_out(f'取消所有的已选中选项', log)
            return self

        except BaseException as e:
            self._err_out(f'取消所有的已选中选项:: { e }')

    def deselect_by_index(self, index: int, eid: int = 0, log: bool = True):
        """
        [ 下拉框中通过下标取消已选中选项 ]
        * 无

        [ 必要参数 ]
        * index (int) : 需要取消选中的选项下标

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._init_select_handle(eid)
            self._select_handle.select_by_index(index)
            select_option_text = self._select_handle.first_selected_option.text
            self._select_handle.deselect_by_index(index)
            self._log_out(f'通过下标取消已选中选项[{ select_option_text }]', log)
            return self

        except BaseException as e:
            self._err_out(f'通过下标取消已选中选项:: { e }')

    def deselect_by_value(self, value: str, eid: int = 0, log: bool = True):
        """
        [ 下拉框中通过选项标签属性值取消已选中选项 ]
        * 无

        [ 必要参数 ]
        * value (int): 需要取消选中的选项标签属性值

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._init_select_handle(eid)
            self._select_handle.select_by_value(value)
            select_item_text = self._select_handle.first_selected_option.text
            self._select_handle.deselect_by_value(value)
            self._log_out(f'通过选项标签属性值取消已选中选项[{ select_item_text }]', log)
            return self

        except BaseException as e:
            self._err_out(f'通过选项标签属性值取消已选中选项:: { e }')

    def deselect_by_text(self, text: str, eid: int = 0, log: bool = True):
        """
        [ 下拉框中通过选项的文本取消已选中选项 ]
        * 无

        [ 必要参数 ]
        * text (int): 需要取消选中的选项的文本

        [ 可选参数 ]
        * eid (int) : 指定元素下标，默认为 0
        * log (bool) : 日志开关

        [ 返回参数 ]
        * 无

        """

        try:
            self._init_select_handle(eid)
            self._select_handle.select_by_visible_text(text)
            select_item_text = self._select_handle.first_selected_option.text
            self._select_handle.deselect_by_visible_text(text)
            self._log_out(f'通过选项的文本取消已选中选项[{ select_item_text }]', log)
            return self

        except BaseException as e:
            self._err_out(f'通过选项的文本取消已选中选项::{ e }')

    def _init_select_handle(self, eid):
        """
        [ 初始化下拉框处理器 ]
        * 无

        [ 必要参数 ]
        * eid (int): 指定元素下标

        [ 可选参数 ]
        * 无

        [ 返回参数 ]
        * 无

        """

        self._element_container = self._analytical_elements()
        self._select_handle = Select(self._element_container[eid])
